# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2005 Christian Mauduit <ufoot@ufoot.org>

from time import asctime

from mod_python import util

from itools import uri

import db
import auth

class Context:
    """This class is used to determine the context of a request,
    and store informations such as its path, its request
    parameters, session related stuff, and so on."""
    def __init__(self,req,cnf):
        """A context is constructed with a request and a config.
        It holds the informations that are usefull to construct
        any page, independently of its being an html, a xul page,
        whatever. Could even have been a parent of all pages, but
        it's implemented as a separated class, to be loaded as
        a member."""
        self.request=req

## http://atlassian01.contegix.com:10083/secure/attachment/14216/content.txt

## The mod_python.publisher implementation always decodes form parameters
## regardless of whether the method being called can accept any and also ignores
## whether the content type is even of a type where form parameters can be
## decoded in the first place. This means that it is not possible using the
## mod_python.publisher module to implement a method which accepts POST
## requests with a inbound content type such as "text/xml". Attempting to do
## so yields an error such as:

## <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
## <html><head>
## <title>501 Method Not Implemented</title>
## </head><body>
## <h1>Method Not Implemented</h1>
## <p>POST to /xmlrpc/service not supported.<br />
## </p>
## <hr />
## <address>Apache/2.0.51 (Unix) mod_python/3.1.4 Python/2.3 Server at localhost Port 8080</address>
## </body></html>

## The "Method Not Implemented" error in this case is due to FieldStorage
## rejecting the content type of "text/xml".

## A check could be added to ensure that FieldStorage is only used to decode
## form parameters if the content type is appropriate. Ie.,

## if not req.headers_in.has_key("content-type"):
##       content_type = "application/x-www-form-urlencoded"
##     else:
##       content_type = req.headers_in["content-type"]

## if content_type == "application/x-www-form-urlencoded" or \
##         content_type[:10] == "multipart/":

## req.form = util.FieldStorage(req,keep_blank_values=1)

## Because req.form is passed to util.apply_fs_data(), code where it is called
## would need to be changed to pass None instead and util.apply_fs_data()
## would need to be modified to see that the argument is None and not do
## the field conversion process of stuff in the form etc etc.

        if not req.headers_in.has_key("content-type"):
            content_type = "application/x-www-form-urlencoded"
        else:
            content_type = req.headers_in["content-type"]

        if content_type == "application/x-www-form-urlencoded" or content_type[:10] == "multipart/":
            self.form = util.FieldStorage(req)
        else:
            self.form = {} # to avoid "has no attribute form" errors

## http://atlassian01.contegix.com:10083/secure/attachment/14216/content.txt

        self.config=cnf
        self.uri=uri.get_reference(self.request.unparsed_uri)
        self.path=self.uri.path
        self.db=db.Db(cnf.params["db_name"],
                      cnf.params["db_host"],
                      cnf.params["db_port"],
                      cnf.params["db_user"],
                      cnf.params["db_passwd"])
        self.auth=auth.Auth(self.db,
                            login=self.get_form_value("login"),
                            passwd=self.get_form_value("passwd"),
                            user_id=self.get_form_value("user_id"),
                            user_passwd=self.get_form_value("user_passwd"))
        if cnf.params["debug_log"]:
            self.debug_log=open(cnf.params["debug_log"],"a")
        else:
            self.debug_log=None
    def get_form_value(self,key,def_value=""):
        """Returns the value of a form field, if value doesn't
        exists, returns a default value ("")."""
        return self.form.has_key(key) and self.form[key] or def_value
    def log(self,str):
        """Logs a message to some custom log file."""
        if self.debug_log:
            self.debug_log.write("%s\n%s:\n %s\n" % (72*"*",asctime(),str))
            self.debug_log.flush()

