# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2005 Christian Mauduit <ufoot@ufoot.org>

import sys
import re
import traceback
import mod_python.apache
from itools.handlers import get_handler
from itools import xml # important to import this *after* get_handler
from itools import uri
import context
import namespaces
import js

def handler(req,cnf):
    """This is the global handler, it traps everything that
    comes in. We then use itools uri parser to determine
    what to do and what to show."""
    ctx=context.Context(req,cnf)
    try:
        # Following route could be put outside handler and
        # executed less often but it is safer to recalculate
        # it each time - and it's not so slow - since Path isn't
        # immutable AFAIK.
        root_path=uri.Path(cnf.params["root"])
        page=root_path.get_pathto(ctx.path)
        namespace=None
        template=None
        str_page=str(page)
        if str_page=="." or not str_page:
            str_page="index.html"
        # For security reasons, we check every possible name
        # page "manually" and then call appropriate code. This
        # way if someone forges an impossible URL, we can handle
        # it properly and avoid some security holes.
        if (str_page=="index.html"):
            namespace=namespaces.index.IndexHtml(ctx)
        elif (str_page=="test.html"):
            namespace=namespaces.test.TestHtml(ctx)
        elif (str_page=="catalog.html"):
            namespace=namespaces.catalog.CatalogHtml(ctx)
        elif (str_page=="view.html"):
            namespace=namespaces.view.ViewHtml(ctx)
        elif (str_page=="google.html"):
            namespace=namespaces.google.GoogleHtml(ctx)
        elif (str_page=="edit.xul"):
            namespace=namespaces.edit.EditXul(ctx)
        elif (str_page=="route.rdf"):
            namespace=namespaces.route.RouteRdf(ctx)
        elif (str_page=="browser.rdf"):
            namespace=namespaces.browser.BrowserRdf(ctx)
        elif (str_page=="service.soap"):
            namespace=namespaces.service.ServiceSoap(ctx)
        elif (str_page=="export.gpx"):
            namespace=namespaces.export.ExportGpx(ctx)
        elif (str_page=="export.kml"):
            namespace=namespaces.export.ExportKml(ctx)
        elif (str_page=="consts.js"):
            namespace=namespaces.consts.ConstsJs(ctx)
        if namespace:
            page_re=re.compile("^(.*)\.(html|xul|rdf|soap|gpx|kml)")
            page_search=page_re.search(str_page)
            if page_search:
                page_name=page_search.groups()[0]
                page_ext=page_search.groups()[1]
                if page_ext=="html":
                    req.content_type = "text/html"
                elif page_ext=="xul":
                    req.content_type = "application/vnd.mozilla.xul+xml"
                elif page_ext=="rdf" or page_ext=="soap" or page_ext=="gpx":
                    req.content_type = "text/xml"
                elif page_ext=="kml":
                    req.content_type = "application/vnd.google-earth.kml+xml"
                template_filename="%s/%s/%s.xml" % (cnf.params["templates"],page_ext,page_name)
                template=get_handler(template_filename)
                # The following route actually builds the XML output
                output=template.stl(namespace.ns)
                #ctx.log(output)
                req.write(output)
            elif str_page=="consts.js":
                req.write(js.consts.src(namespace.ns))
            else:
                # TODO: handle error here
                req.content_type = "text/plain"
                req.write("namespace defined but page doesn't exist")
                pass
        else:
            # TODO: handle error here
            req.content_type = "text/plain"
            req.write("page doesn't exist, no namespace")
            pass
    except Exception:
        trace="%s\n%s\n%s" % (sys.exc_type, sys.exc_value, "".join(traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
        ctx.log(trace)
        req.write(trace)
    return mod_python.apache.OK
